@@ -82,7 +82,7 @@ unsigned int IonColourFilter::refresh(const std::vector<const FilterStreamData *
 	if(cacheOK)
 	{
 		ASSERT(filterOutputs.size());
-		propagateStreams(dataIn,getOut,STREAM_TYPE_IONS,false);
+		propagateStreams(dataIn,getOut,getRefreshBlockMask(),true);
 
 		propagateCache(getOut);
 
@@ -363,7 +363,9 @@ bool IonColourFilter::setProperty(  unsigned int key,
 		case KEY_IONCOLOURFILTER_MAPEND:
 		{
 			float tmpBound;
-			stream_cast(tmpBound,value);
+			if(stream_cast(tmpBound,value))
+				return false;
+
 			if(tmpBound <=mapBounds[0])
 				return false;
 
