@@ -906,8 +906,8 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 	//----
 	//TODO: Other filtering? threshold/median? laplacian? etc
 	
-	choices.clear();
 /*
+	choices.clear();
 	//Post-filtering method
 	for(unsigned int ui=0;ui<VOXELISE_FILTERTYPE_MAX; ui++)
 	{
@@ -952,9 +952,9 @@ void VoxeliseFilter::getProperties(FilterPropGroup &propertyList) const
 		propertyList.addProperty(p,curGroup);
 	}
 	propertyList.setGroupTitle(curGroup,TRANS("Filtering"));
-	curGroup++;*/
+	curGroup++;
 	//----
-
+*/
 	//start a new group for the visual representation
 	//----------------------------
 	choices.clear();
@@ -1460,10 +1460,14 @@ bool VoxeliseFilter::setProperty(unsigned int key,
 		}
 		case KEY_FILTER_MODE:
 		{
+			//Locate the current string
 			unsigned int i;
-			for (i = 0; i < VOXEL_REPRESENT_END; i++)
-				if (value == getFilterTypeString(i)) break;
-			if (i == VOXEL_REPRESENT_END)
+			for (i = 0; i < VOXELISE_FILTERTYPE_MAX; i++)
+			{
+				if (value == getFilterTypeString(i)) 
+					break;
+			}
+			if (i == VOXELISE_FILTERTYPE_MAX)
 				return false;
 			if(i!=filterMode)
 			{
