@@ -668,9 +668,16 @@ unsigned int FilterTree::refreshFilterTree(list<FILTER_OUTPUT_DATA > &outData,
 						currentFilter->setCaching(false);
 						break;
 					case CACHE_DEPTH_FIRST:
-						currentFilter->setCaching(cacheBytes/(1024*1024) < maxCachePercent*getAvailRAM());
-						break;
+					{
+						float ramFreeForUse;
+						ramFreeForUse= maxCachePercent/(float)100.0f*getAvailRAM();
 
+						bool cache;
+						cache=(cacheBytes/(1024*1024) ) < ramFreeForUse;
+
+						currentFilter->setCaching( cache);
+						break;
+					}
 				}
 			}
 			else
@@ -888,6 +895,77 @@ bool FilterTree::setFilterProperty(Filter *targetFilter, unsigned int key,
 
 }
 
+void FilterTree::serialiseToStringPaths(std::map<const Filter *, string > &serialisedPaths) const
+{
+
+	stack<string> pathStack;
+	pathStack.push("");
+
+	set<string> enumeratedPaths;
+
+	//Unlikely text string that can be appended to tree path
+	const char *PATH_NONCE="$>";
+
+	unsigned int lastDepth=0;
+	for(tree<Filter *>::iterator filterIt=filters.begin();
+			filterIt!=filters.end();++filterIt)
+	{
+		//if this is a new depth, pop the stack until
+		// we hit the correct level
+		unsigned int curDepth;
+		curDepth=depth(filterIt);
+		//Add one for base element
+		while(pathStack.size() > curDepth +1)
+		{
+			pathStack.pop();
+			lastDepth--;
+		}
+
+		std::string testPath;
+		testPath = pathStack.top() + string("/")  + (*filterIt)->typeString();
+
+		unsigned int nonceIncrement;
+		nonceIncrement=0;
+		while(enumeratedPaths.find(testPath) != enumeratedPaths.end())
+		{
+			std::string tailStr;
+			nonceIncrement++;
+			stream_cast(tailStr,nonceIncrement);
+
+			//Keep trying new path with nonce
+			testPath=pathStack.top()+(*filterIt)->typeString() + string(PATH_NONCE) + tailStr;
+		}
+
+		enumeratedPaths.insert(testPath);
+		const Filter *f;
+		f=(const Filter*)(*filterIt);
+		serialisedPaths.insert(make_pair(f,testPath));
+
+		pathStack.push(testPath);
+
+	}
+
+	ASSERT(serialisedPaths.size() == filters.size());
+}
+
+
+void FilterTree::serialiseToStringPaths(map<string, const Filter * > &serialisedPaths) const
+{
+	//Build one-way mapping
+	map<const Filter *, string> singleMap;
+	serialiseToStringPaths(singleMap);
+
+
+	serialisedPaths.clear();
+	for(map<const Filter*,string>::iterator it=singleMap.begin();
+		it!=singleMap.end();++it)
+	{
+		ASSERT(serialisedPaths.find(it->second) == serialisedPaths.end());
+		serialisedPaths[it->second]=it->first;	
+	}
+
+}
+
 unsigned int FilterTree::loadXML(const xmlNodePtr &treeParent, std::ostream &errStream,const std::string &stateFileDir)
 
 {
@@ -1107,7 +1185,8 @@ bool FilterTree::saveXML(std::ofstream &f,std::map<string,string> &fileMapping,
 
 	return true;
 }
-		
+	
+
 
 bool FilterTree::hasHazardousContents() const
 {
@@ -1347,6 +1426,8 @@ void FilterTree::safeDeleteFilterList( std::list<FILTER_OUTPUT_DATA> &outData,
 	for(list<FILTER_OUTPUT_DATA> ::iterator it=outData.begin(); 
 							it!=outData.end(); ) 
 	{
+		vector<bool> killV;
+		killV.resize(it->second.size(),false);
 		//Note the No-op at the loop iterator. this is needed so we can safely .erase()
 		for(size_t ui=0;ui<it->second.size();ui++)
 		{
@@ -1363,10 +1444,12 @@ void FilterTree::safeDeleteFilterList( std::list<FILTER_OUTPUT_DATA> &outData,
 			if(!f->cached)
 				delete f;
 	
-			std::swap(f,it->second.back());
-			it->second.pop_back();
+			killV[ui]=true;
 		}
 
+
+		vectorMultiErase(it->second,killV);
+
 		//Check to see if this element still has any items in its vector. if not,
 		//then discard the element
 		if(!(it->second.size()))
@@ -1776,7 +1859,18 @@ void FilterTree::clearCacheByType(unsigned int type)
 
 }
 
+size_t FilterTree::cacheCount(unsigned int typeMask) const
+{
+	size_t count=0;
+	for(tree<Filter * >::iterator it=filters.begin(); 
+					it!=filters.end(); ++it)
+	{
+		if(((*it)->getType() & typeMask) && (*it)->haveCache())
+			count++;
+	}
 
+	return count;
+}
 		
 void FilterTree::modifyRangeFiles(const map<const RangeFile *, const RangeFile *> &toModify)
 {
