@@ -61,8 +61,6 @@ class FilterTree
 		void getFilterRefreshStarts(vector<tree<Filter *>::iterator > &propStarts) const;
 		
 			
-		//Retrieve the filter's pointer from its ID value	
-		Filter* getFilterByIdNonConst(unsigned long long filterId) const;
 		
 #ifdef DEBUG
 		//!Check that the output of filter refresh functions
@@ -154,6 +152,12 @@ class FilterTree
 					bool writePackage, bool useRelativePaths, unsigned int minTabDepth=0) const;
 
 
+		//!Convert tree to a series of flat strings representing the topology
+		//TODO: COnvert to bimap
+		void serialiseToStringPaths(std::map<const Filter *,string > &serialisedPaths) const;
+		void serialiseToStringPaths(std::map<string,const Filter *> &serialisedPaths) const;
+
+
 		//Topological alteration  & examination functions
 		//----------	
 		//!Remove an element and all sub elements from the tree, 
@@ -222,6 +226,9 @@ class FilterTree
 		//Overwrite the contents of the pointed-to range files with
 		// the map contents
 		void modifyRangeFiles(const map<const RangeFile *, const RangeFile *> &toModify);
+
+
+		size_t cacheCount(unsigned int typeMask = STREAMTYPE_MASK_ALL) const;
 };
 
 #endif
