@@ -50,6 +50,13 @@ enum{
 	BOUND_ENUM_END
 };
 
+//Interpolation mode for slice 
+enum
+{
+	SLICE_INTERP_NONE,
+	SLICE_INTERP_LINEAR,
+	SLICE_INTERP_ENUM_END
+};
 
 enum{
 	ADJ_ALL,
@@ -117,6 +124,9 @@ template<class T> class Voxels
 	
 		bool (*callback)(bool);
 			
+
+		void localPaddedConvolve(long long ui,long long uj, long long uk, 
+			const Voxels<T> &kernel,Voxels<T> &result, unsigned int mode) const;
 	public:
 		//!Constructor.
 		Voxels();
@@ -163,7 +173,15 @@ template<class T> class Voxels
 		void setData(size_t x, size_t y, size_t z, const T &val);
 		//!Set the value of nth point in the dataset
 		void setData(size_t n, const T &val);
-		
+
+
+		//get an interpolated slice from a section of the data
+		void getSlice(size_t normal, float offset, T *p, 
+			size_t interpMode=SLICE_INTERP_NONE, size_t boundMode=BOUND_MIRROR) const;
+
+		//Get a specific slice, from an integral offset in the data
+		void getSlice(size_t normal, size_t offset, 
+					T *p,size_t boundMode) const;
 		//!Get the size of the data field
 		void getSize(size_t &x, size_t &y, size_t &z) const;
 		size_t getSize() const {return voxels.size();};
@@ -228,10 +246,14 @@ template<class T> class Voxels
 		//!Get the bounding size
 		Point3D getMinBounds() const;
 		Point3D getMaxBounds() const;
+		//Obtain the ounds for a specified axis
+		void getAxisBounds(size_t axis, float &minV, float &maxV) const;
 		///! Get the spacing for a unit cell
 		Point3D getPitch() const;
 		//!Set the bounding size
 		void setBounds(const Point3D &pMin, const Point3D &pMax);
+		//!Get the bounding size
+		void getBounds(Point3D &pMin, Point3D &pMax) const { pMin=minBound;pMax=maxBound;}
 
 		//!Initialise the voxel storage
 		size_t init(size_t nX,size_t nY,size_t nZ, const BoundCube &bound);
@@ -340,7 +362,7 @@ template<class T> class Voxels
 		int histogram(std::vector<size_t> &v, size_t histBinCount) const;
 
 		//!Find the largest or smallest n objects
-		void findSorted(std::vector<size_t> &x, std::vector<size_t> &y, std::vector<size_t> &z, size_t n, bool largest=true) const;
+		void findNExtrema(std::vector<size_t> &x, std::vector<size_t> &y, std::vector<size_t> &z, size_t n, bool largest=true) const;
 
 		//!Generate a dataset that consists of the counts of points in a given voxel
 		/*! Ensure that the voxel scaling factors 
@@ -516,7 +538,7 @@ void Voxels<T>::getSize(size_t &x, size_t &y, size_t &z) const
 template<class T>
 size_t Voxels<T>::getEdgeIndex(size_t x,size_t y, size_t z, unsigned int index) const
 {
-	//This provides a reversable mapping of x,y,z 
+	//This provides a reversible mapping of x,y,z 
 	//X aligned edges are first
 	//Y second
 	//Z third
@@ -861,6 +883,13 @@ void Voxels<T>::getEdgeEndApproxVals(size_t edgeUniqId, float &a, float &b ) con
 
 }
 
+template<class T>
+void Voxels<T>::getAxisBounds(size_t axis, float &minV, float &maxV ) const
+{
+	maxV=maxBound[axis];
+	minV=minBound[axis];
+}
+
 template<class T>
 size_t Voxels<T>::resize(size_t x, size_t y, size_t z, const Point3D &newMinBound, const Point3D &newMaxBound) 
 {
@@ -1239,21 +1268,55 @@ void Voxels<T>::swap(Voxels<T> &kernel)
 	std::swap(minBound,kernel.minBound);
 }
 
+//Helper function for performing convolution
 template<class T>
-size_t Voxels<T>::convolve(const Voxels<T> &kernel, Voxels<T> &result,  size_t boundMode) const
+void Voxels<T>::localPaddedConvolve(long long ui,long long uj, long long uk, 
+			const Voxels<T> &kernel,Voxels<T> &result, unsigned int mode) const
 {
-	ASSERT(voxels.size());
-	ASSERT(callback);
+	ASSERT(result.binCount[0] == binCount[0] && result.binCount[1]==binCount[1]
+		&& result.binCount[2] == binCount[2]);
 
-	//Check the kernel can fit within this datasest
+	long long halfX,halfY,halfZ;
+#ifdef DEBUG
+	for(size_t i=0;i<3;i++)
+	{
+		ASSERT(kernel.binCount[i] &1);
+	}
+#endif
+	halfX=kernel.binCount[0]/2; halfY=kernel.binCount[1]/2;halfZ=kernel.binCount[2]/2;
+
+	T tally(0.0);
+	for(size_t ul=0; ul<kernel.binCount[0]; ul++)
+	{
+	for(size_t um=0; um<kernel.binCount[1]; um++)
+	{
+	for(size_t un=0; un<kernel.binCount[2]; un++)
+	{
+		tally+=getPaddedData(ui+ul-halfX,uj+um-halfY,uk+un-halfZ,mode)*
+				kernel.getData(ul,um,un);
+	}
+	}
+	}
+
+	result.setData(ui,uj,uk,tally);	
+}
+
+template<typename T>
+size_t Voxels<T>::convolve(const Voxels<T> &kernel, Voxels<T> &result,  size_t boundMode)  const
+{
+
+	//Check the kernel can fit within this est
 	size_t x,y,z;
 	kernel.getSize(x,y,z);
 	if(binCount[0] <x || binCount[1]< y || binCount[2] < z)
 		return 1;
+	
+	long long halfX,halfY,halfZ;
+	halfX=kernel.binCount[0]/2; halfY=kernel.binCount[1]/2;halfZ=kernel.binCount[2]/2;
 
 	//Loop through all of the voxel elements, setting the
 	//result voxels to the convolution of the template
-	//with the data 
+	//with the data
 
 
 	switch(boundMode)
@@ -1275,8 +1338,9 @@ size_t Voxels<T>::convolve(const Voxels<T> &kernel, Voxels<T> &result,  size_t b
 			if(result.resize(sX,sY,sZ,resultMinBound, resultMaxBound))
 				return VOXELS_OUT_OF_MEMORY;
 
-			//loop through each data element
+			//loop through each element
 			//Forgive the poor indenting, but the looping is very deep
+			//FIXME: there is a shift in the output when doing this?
 			for (size_t ui=0;ui<(size_t)(binCount[0]-x+1); ui++)
 			{
 			for (size_t uj=0;uj<binCount[1]-y+1; uj++)
@@ -1304,12 +1368,130 @@ size_t Voxels<T>::convolve(const Voxels<T> &kernel, Voxels<T> &result,  size_t b
 			}
 			
 			}
+			break;
+		}
+		case BOUND_MIRROR:
+		case BOUND_HOLD:
+		case BOUND_DERIV_HOLD:
+		{
+			//Calculate the bounding size of the resultant box
+			//Set up the result box
+			if(result.resize(binCount[0],binCount[1],binCount[2],minBound,maxBound))
+				return VOXELS_OUT_OF_MEMORY;
+
+			//loop through each element in the interior
+			//Forgive the poor indenting, but the looping is very deep
+			{
+			for (long long ui=halfX;ui<(size_t)(binCount[0]-x+1); ui++)
+			{
+			for (long long uj=halfY;uj<binCount[1]-y+1; uj++)
+			{
+			for (long long uk=halfZ;uk<binCount[2]-z+1; uk++)
+			{
+				T tally;
+				tally=T(0.0);
+
+				//Convolve this element with the kernel 
+				for(long long ul=0; ul<x; ul++)
+				{
+				for(long long um=0; um<y; um++)
+				{
+				for(long long un=0; un<z; un++)
+				{
+					tally+=(getData(ui+ ul-halfX,uj + um -halfY,uk + un -halfZ))*
+							kernel.getData(ul,um,un);
+				}
+				}
+				}
+
+				result.setData(ui,uj,uk,tally);	
+			}
+			}
+			}
+			}
+
+			//Lower Boundaries
+			//----------
+			//x- boundary
+			for(long long ui=0;ui<halfX;ui++)
+			{
+			for(long long uj=0;uj<binCount[1];uj++)
+			{
+			for(long long uk=0;uk<binCount[2];uk++)
+			{
+				localPaddedConvolve(ui,uj,uk,kernel,result,boundMode);
+			}
+			}
+			}
+			
+			//y- boundary
+			for(long long ui=0;ui<binCount[0];ui++)
+			{
+			for(long long uj=0;uj<halfY;uj++)
+			{
+			for(long long uk=0;uk<binCount[2];uk++)
+			{
+				localPaddedConvolve(ui,uj,uk,kernel,result,boundMode);
+			}
+			}
+			}
+			
+			//z- boundary
+			for(long long ui=0;ui<binCount[0];ui++)
+			{
+			for(long long uj=0;uj<binCount[1];uj++)
+			{
+			for(long long uk=0;uk<halfZ;uk++)
+			{
+				localPaddedConvolve(ui,uj,uk,kernel,result,boundMode);
+			}
+			}
+			}
+			//----------
+	
+			//Upper boundary
+			//----------
+			//x+ boundary
+			for(long long ui=binCount[0]-x+1;ui<binCount[0];ui++)
+			{
+			for(long long uj=halfY;uj<binCount[1];uj++)
+			{
+			for(long long uk=halfZ;uk<binCount[2];uk++)
+			{	
+				localPaddedConvolve(ui,uj,uk,kernel,result,boundMode);
+			}
+			}
+			}
+
+			//y+ boundary
+			for(long long ui=halfX;ui<binCount[0];ui++)
+			{
+			for(long long uj=binCount[1]-y+1;uj<binCount[1];uj++)
+			{
+			for(long long uk=halfZ;uk<binCount[2];uk++)
+			{
+				localPaddedConvolve(ui,uj,uk,kernel,result,boundMode);
+			}
+			}
+			}
+			
+			//z+ boundary
+			for(long long ui=halfX;ui<binCount[0];ui++)
+			{
+			for(long long uj=halfY;uj<binCount[1];uj++)
+			{
+			for(long long uk=binCount[2]-z+1;uk<binCount[2];uk++)
+			{
+				localPaddedConvolve(ui,uj,uk,kernel,result,boundMode);
+			}
+			}
+			}
+			//----------
+			
+
 
 			break;
 		}
-		case BOUND_HOLD:
-		case BOUND_DERIV_HOLD:
-		case BOUND_MIRROR:
 		default:
 			ASSERT(false);
 			return 2;
@@ -1966,7 +2148,7 @@ void Voxels<T>::makeSphericalKernel(size_t sideLen, float bound, const T &val, u
 						}
 
 						//Level 0 corresponds to 1/8th of the original voxel. Level n = 1/(2^3(n+1)) 
-						//each voxel has side lenght L_v = originalLen/(2^(level+1))
+						//each voxel has side length L_v = originalLen/(2^(level+1))
 						while(!positionStack.empty())
 						{
 							unsigned int thisLevel;
@@ -2452,10 +2634,10 @@ int Voxels<T>::histogram(std::vector<size_t> &v, size_t histBinCount) const
 }
 
 template<class T>
-void Voxels<T>::findSorted(std::vector<size_t> &x, std::vector<size_t> &y, 
+void Voxels<T>::findNExtrema(std::vector<size_t> &x, std::vector<size_t> &y, 
 			std::vector<size_t> &z, size_t n, bool largest) const
 {
-	//Could be better if we didn't use indexed data aquisition (record opsition)
+	//Could be better if we didn't use indexed data acquisition (record position)
 	std::deque<size_t> bSx,bSy,bSz;
 
 	if(voxels.empty())
@@ -2538,6 +2720,168 @@ void Voxels<T>::findSorted(std::vector<size_t> &x, std::vector<size_t> &y,
 	}
 }
 
+
+//Obtain a slice of the voxel data. Data output will be in column order
+// p[posB*nA + posA]. Input slice must be sufficiently sized and allocated
+// to hold the output data
+template<class T>
+void Voxels<T>::getSlice(size_t normalAxis, size_t offset, T *p,size_t boundMode) const
+{
+	ASSERT(normalAxis < 3);
+
+	size_t dimA,dimB,nA;
+	switch(normalAxis)
+	{
+		case 0:
+		{
+			dimA=1;
+			dimB=2;
+			nA=binCount[dimA];
+			break;
+		}
+		case 1:
+		{	
+			dimA=0;
+			dimB=2;
+			nA=binCount[dimA];
+			break;
+		}
+		case 2:
+		{
+			dimA=0;
+			dimB=1;
+			nA=binCount[dimA];
+			break;
+		}
+		default:
+			ASSERT(false); //WTF - how did you get here??
+	}
+		
+
+	if(offset < binCount[normalAxis])
+	{
+		//We are within bounds, use normal access functions
+		switch(normalAxis)
+		{
+			case 0:
+			{
+				for(size_t ui=0;ui<binCount[dimA];ui++)
+				{
+					for(size_t uj=0;uj<binCount[dimB];uj++)
+						p[uj*nA + ui] =	getData(offset,ui,uj);
+				}
+				break;
+			}
+			case 1:
+			{	
+				for(size_t ui=0;ui<binCount[dimA];ui++)
+				{
+					for(size_t uj=0;uj<binCount[dimB];uj++)
+						p[uj*nA + ui] =	getData(ui,offset,uj);
+				}
+				break;
+			}
+			case 2:
+			{
+				for(size_t ui=0;ui<binCount[dimA];ui++)
+				{
+					for(size_t uj=0;uj<binCount[dimB];uj++)
+						p[uj*nA + ui] =	getData(ui,uj,offset);
+				}
+				break;
+			}
+			default:
+				ASSERT(false);
+		}
+	}
+	else
+	{
+		//We are outside the cube bounds, use the padded access functions
+		switch(normalAxis)
+		{
+			case 0:
+			{
+				for(size_t ui=0;ui<binCount[dimA];ui++)
+				{
+					for(size_t uj=0;uj<binCount[dimB];uj++)
+						p[uj*nA + ui] =	getPaddedData(offset,ui,uj,boundMode);
+				}
+				break;
+			}
+			case 1:
+			{	
+				for(size_t ui=0;ui<binCount[dimA];ui++)
+				{
+					for(size_t uj=0;uj<binCount[dimB];uj++)
+						p[uj*nA + ui] =	getPaddedData(ui,offset,uj,boundMode);
+				}
+				break;
+			}
+			case 2:
+			{
+				for(size_t ui=0;ui<binCount[dimA];ui++)
+				{
+					for(size_t uj=0;uj<binCount[dimB];uj++)
+						p[uj*nA + ui] =	getPaddedData(ui,uj,offset,boundMode);
+				}
+				break;
+			}
+			default:
+				ASSERT(false);
+		}
+	}
+}
+
+template<class T>
+void Voxels<T>::getSlice(size_t normal, float offset, 
+		T *p, size_t interpMode,size_t boundMode) const
+{
+	ASSERT(offset <=1.0f && offset >=0.0f);
+
+	//Obtain the appropriately interpolated slice
+	switch(interpMode)
+	{
+		case SLICE_INTERP_NONE:
+		{
+			size_t slicePos;
+			slicePos=roundf(offset*binCount[normal]);
+			getSlice(normal,slicePos,p,boundMode);
+			break;
+		}
+		case SLICE_INTERP_LINEAR:
+		{
+			//Find the upper and lower bounds
+			size_t sliceUpper,sliceLower;;
+			sliceUpper=ceilf(offset*binCount[normal]);
+			sliceLower=offset*binCount[normal];
+
+			{
+				T *pLower;
+				size_t numEntries=binCount[(normal+1)%3]*binCount[(normal+2)%3];
+				
+				pLower  = new T[numEntries];
+
+				getSlice(normal,sliceLower,pLower,boundMode);
+				getSlice(normal,sliceUpper,p,boundMode);
+
+				//Get the decimal part of the float
+				float integ;
+				float delta=modff(offset*binCount[normal],&integ);
+				for(size_t ui=0;ui<numEntries;ui++)
+					p[ui] = delta*(p[ui]-pLower[ui]) + pLower[ui];
+
+				delete[] pLower;
+			}
+			break;
+		}
+		default:
+			ASSERT(false);
+			
+	}
+
+}
+
+
 template<class T>
 void Voxels<T>::operator/=(const Voxels<T> &v)
 {
