@@ -48,6 +48,7 @@ Scene::Scene() : tempCam(0), cameraSet(true), outWinAspect(1.0f)
 	showAxis=true;
 	attemptedLoadProgressAnim=false;
 	showProgressAnimation=false;
+	witholdCamUpdate=false;
 
 	//default to black
 	rBack=gBack=bBack=0.0f;
@@ -68,6 +69,7 @@ Scene::~Scene()
 {
 	clearAll();
 	DrawableObj::clearTexPool();
+	delete activeCam;
 }
 
 unsigned int Scene::initDraw()
@@ -641,12 +643,26 @@ void Scene::setAspect(float newAspect)
 {
 	outWinAspect=newAspect;
 }
+void Scene::setActiveCam(Camera *c)
+{
+	if(tempCam)
+		discardTempCam();
+
+	if(activeCam)
+		delete activeCam;
+
+	activeCam = c;
+	cameraSet=true;
+}
 
 void Scene::setActiveCamByClone(const Camera *c)
 {
 	if(tempCam)
 		discardTempCam();
 
+	if(activeCam)
+		delete activeCam;
+	
 	activeCam = c->clone();
 	cameraSet=true;
 }
