@@ -201,7 +201,10 @@ class Scene
 		//!remove a drawable object
 		void removeDrawable(unsigned int);
 
-		void setActiveCam();
+		//!Set the active camera directly
+		// note that the pointer becomes "owned" by the scene.
+		// any previous active camera will be deleted
+		void setActiveCam(Camera *c);
 		//! set the active camera
 		void setActiveCamByClone(const Camera *c);
 
@@ -333,6 +336,8 @@ class Scene
 
 		//!Clear effects vector
 		void clearEffects();
+
+		static string getGlVersion() { return  string((char *)glGetString(GL_VERSION)); }
 };
 
 #endif
