@@ -98,6 +98,19 @@ public:
 
     //!Obtain the format the user wants to save ranges in
     size_t getRangeFormat()  const;
+
+    //! Obtain the current state from the animation (keyframes)
+    // the second element provides the mappings for the property animator to 
+    // filter tree path locations
+    void getAnimationState(PropertyAnimator &prop,  vector<pair<string,size_t> > &pathMapping) const ;
+    //!Obtain the current state from the animation
+    void setAnimationState(const PropertyAnimator &prop,
+		    const vector<pair<string,size_t> > &pathMapping);
+
+    //!Obtain the filter tree path string->animation ID mapping
+    void getPathMapping(vector<pair<string,size_t> > &mapping, bool allowMissing=false) const;
+
+    void setDefImSize(unsigned int w, unsigned int h) ; 
 private:
     //!Tree of filters that can be manipulated
     const FilterTree *filterTree;
@@ -139,6 +152,9 @@ private:
     size_t rangeExportMode;
 
 
+    //viewport aspect ratio for image output
+    float imageAspectRatio;
+
     //Used to jump out of wx events that are generated by 
     // the code, rather than the user, eg text events
     bool programmaticEvent;
@@ -214,7 +230,6 @@ public:
     virtual void OnCheckOutDataChange(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnCheckImageOutput(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnImageFilePrefix(wxCommandEvent &event); // wxGlade: <event_handler>
-    virtual void OnTextImageSize(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnBtnResolution(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnCheckPointOutput(wxCommandEvent &event); // wxGlade: <event_handler>
     virtual void OnCheckPlotOutput(wxCommandEvent &event); // wxGlade: <event_handler>
